/* errwarn.c */

#include "fumied.h"
#include <errno.h>

static void do_percentm(char *obuf, char *ibuf);

static char mbuf [1024];
static char fbuf [1024];

/* Log an error message, then exit... */

void
error(char *fmt, ...)
{
  va_list list;

  do_percentm (fbuf, fmt);

  va_start (list, fmt);
  vsnprintf (mbuf, sizeof mbuf, fbuf, list);
  va_end (list);

#ifndef DEBUG
  syslog (log_priority | LOG_ERR, mbuf);
#endif

  /* Also log it to stderr? */
  if (log_perror)
    {
      write (2, mbuf, strlen (mbuf));
      write (2, "\n", 1);
    }

  syslog (LOG_CRIT, "exiting.");
  if (log_perror)
    {
      fprintf (stderr, "exiting.\n");
      fflush (stderr);
    }
  cleanup ();
  exit (1);
}

/* Log a warning message... */

int
warn(char *fmt, ...)
{
  va_list list;

  do_percentm (fbuf, fmt);

  va_start (list, fmt);
  vsnprintf (mbuf, sizeof mbuf, fbuf, list);
  va_end (list);

#ifndef DEBUG
  syslog (log_priority | LOG_ERR, mbuf);
#endif

  if (log_perror)
    {
      write (2, mbuf, strlen (mbuf));
      write (2, "\n", 1);
    }

  return 0;
}

/* Log a note... */

int
note(char *fmt, ...)
{
  va_list list;

  do_percentm (fbuf, fmt);

  va_start (list, fmt);
  vsnprintf (mbuf, sizeof mbuf, fbuf, list);
  va_end (list);

#ifndef DEBUG
  syslog (log_priority | LOG_INFO, mbuf);
#endif

  if (log_perror)
    {
      write (2, mbuf, strlen (mbuf));
      write (2, "\n", 1);
    }

  return 0;
}

/* Log a debug message... */

int
debug(char *fmt, ...)
{
  va_list list;

  do_percentm (fbuf, fmt);

  va_start (list, fmt);
  vsnprintf (mbuf, sizeof mbuf, fbuf, list);
  va_end (list);

#ifndef DEBUG
  syslog (log_priority | LOG_DEBUG, mbuf);
#endif

  if (log_perror)
    {
      write (2, mbuf, strlen (mbuf));
      write (2, "\n", 1);
    }

  return 0;
}

/* Find %m in the input string and substitute an error message string. */

static void
do_percentm(char *obuf, char *ibuf)
{
  char *s = ibuf;
  char *p = obuf;
  int infmt = 0;
  char *m;

  while (*s)
    {
      if (infmt)
	{
	  if (*s == 'm')
	    {
	      m = strerror (errno);
	      if (!m)
		m = "<unknown error>";
	      strcpy (p - 1, m);
	      p += strlen (p);
	      ++s;
	    }
	  else
	    *p++ = *s++;
	  infmt = 0;
	}
      else
	{
	  if (*s == '%')
	    infmt = 1;
	  *p++ = *s++;
	}
    }
  *p = 0;
}
