/* fumied.c */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/utsname.h>
#include "fumied.h"

void
fumied(struct packet *packet)
{
  switch (packet -> packet_type)
    {
    case FUMIEDDISCOVER:
      fumieddiscover (packet);
      break;
    
    case FUMIEDREQUEST:
      fumiedrequest (packet);
      break;
    default:
      break;
    }
}

void
fumieddiscover(struct packet *packet)
{
  struct fumiedMessage raw;
  struct sockaddr_in to;
  int result;

  note ("FUMIEDDISCOVER");

  memcpy (&raw, packet->raw, sizeof(raw));
  to.sin_family = AF_INET;
  to.sin_port = htons(FUMIED_CLIENT_PORT);
  to.sin_addr = raw.ciaddr;
  result = send_packet (packet->interface,
			(struct packet *)0, &raw, packet->packet_length,
			raw.siaddr, &to, packet->haddr);
  if (result < 0)
    warn ("sendpkt: %m");

}

void
fumiedrequest(struct packet *packet)
{
  struct fumiedMessage raw;
  struct sockaddr_in to;
  int result;
  char peer_hostname[80];
  char local_hostname[80];
  char peer_ipaddr[80];
  char local_ipaddr[80];
  struct utsname sysname;

  note ("FUMIEDREQUEST");

  memcpy (&raw, packet->raw, sizeof(raw));
  strncpy(peer_hostname, &raw.options[5], sizeof(peer_hostname)); 
  strcpy(peer_ipaddr, inet_ntoa(raw.ciaddr));
	
  note ("peer hostname:%s", peer_hostname);
  note ("peer ipaddr:%s", peer_ipaddr);

  if (!uname(&sysname))
    {
      strcpy(&raw.options[5], sysname.nodename);
      strcpy(local_hostname, sysname.nodename);
    }
  strcpy(local_ipaddr, inet_ntoa(raw.siaddr));

  note ("local hostname:%s", local_hostname);
  note ("local ipaddr:%s", local_ipaddr);

  to.sin_family = AF_INET;
  to.sin_port = htons(FUMIED_CLIENT_PORT);
  to.sin_addr = raw.ciaddr;
  result = send_packet (packet->interface,
			(struct packet *)0, &raw, packet->packet_length,
			raw.siaddr, &to, packet->haddr);
  if (result < 0)
    warn ("sendpkt: %m");
  revise_hosts(local_hostname, local_ipaddr, peer_hostname, peer_ipaddr);
}

void
revise_hosts(char *local_hostname, char *local_ipaddr,
	     char *peer_hostname, char *peer_ipaddr)
{
  FILE *rfp, *wfp;
   char line[80], *p;
  int local_done = 0, peer_done = 0;

  if ((wfp = fopen(temp_hosts, "w")) == NULL )
    {
      return;
    }
  
  if ((rfp = fopen(etc_hosts, "r")) != NULL )
    {
      while (fgets(line, 80, rfp) != NULL)
	{
	  p = strstr(line, local_hostname);
	  if (p == NULL)
	    {
	      if (peer_hostname != NULL)
		{
		  p = strstr(line, peer_hostname);
		  if (p == NULL)
		    {
		      fputs(line, wfp);
		    }
		  else
		    {
		      for (p = line; p != NULL; p++)
			{
			  if (isspace(*p))
			    break;
			}
		      fprintf(wfp, "%s", peer_ipaddr);
		      fprintf(wfp, "%s", p);
		      peer_done = 1;
		    }
		}
	      else
		{
		  fputs(line, wfp);
		}
	    }
	  else
	    {
	      for (p = line; p != NULL; p++)
		{
		  if (isspace(*p))
		    break;
		}
	      fprintf(wfp, "%s", local_ipaddr);
	      fprintf(wfp, "%s", p);
	      local_done = 1;
	    }
	}
      if (local_done == 0)
	{
	  fprintf(wfp, "%s\t%s", local_ipaddr, local_hostname);
	  strcpy(line, local_hostname);
	  if ((p = strchr(line, '.')) != NULL)
	    {
	      *p = '\0';
	      fprintf(wfp, " %s\n", line);
	    }
	  else
	    {
	      fprintf(wfp, "\n");
	    }
	}
      if ((peer_hostname != NULL) && (peer_done == 0))
	{
	  fprintf(wfp, "%s\t%s", peer_ipaddr, peer_hostname);
	  strcpy(line, peer_hostname);
	  if ((p = strchr(line, '.')) != NULL)
	    {
	      *p = '\0';
	      fprintf(wfp, " %s\n", line);
	    }
	  else
	    {
	      fprintf(wfp, "\n");
	    }
	}
      fclose(rfp);
    }
  else
    {
      fprintf(wfp, "%s\t%s", local_ipaddr, local_hostname);
      strcpy(line, local_hostname);
      if ((p = strchr(line, '.')) != NULL)
	{
	  *p = '\0';
	  fprintf(wfp, " %s\n", line);
	}
      else
	{
	  fprintf(wfp, "\n");
	}
      if (peer_hostname != NULL)
	{
	  fprintf(wfp, "%s\t%s", peer_ipaddr, peer_hostname);
	  strcpy(line, peer_hostname);
	  if ((p = strchr(line, '.')) != NULL)
	    {
	      *p = '\0';
	      fprintf(wfp, " %s\n", line);
	    }
	  else
	    {
	      fprintf(wfp, "\n");
	    }
	}
    }

  fclose(wfp);
  rename(temp_hosts, etc_hosts);

  return;
}
