/* socket-if.c */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "fumied.h"
#include "udpipgen.h"

void
openSendSocket(struct sockaddr_in *addr, int *s)
{
  int optval = 1;

  if ( (*s = socket(AF_INET, SOCK_DGRAM, 0)) < 0 )
    {
      error("socket (openSendSocket)");
    }
  if ( setsockopt(*s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval)) < 0)
    {
      error("setsockopt (openSendSocket)");
    }
  /* use FUMIED client port for the source port because
   * some servers do NOT use the FUMIED client port
   * BUT use the UDP source  port number of the received
   * datagram for the destination UDP port when it responds
   * to clients.
   */
  addr->sin_port = htons(FUMIED_CLIENT_PORT);
#if 0
  addr->sin_port = htons(0);	/* appropriate unused port */
#endif
  if ( bind(*s, (struct sockaddr *)addr, sizeof(*addr))  < 0 )
    {
      error("bind (openSendSocket)");
    }
}

void
openRecvSocket(struct sockaddr_in *addr, int *s)
{
  int optval = 1;

  if ( (*s = socket(AF_INET, SOCK_DGRAM, 0)) < 0 )
    {
      error("socket (openRecvSocket)");
    }
  if ( setsockopt(*s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval)) < 0)
    {
      error("setsockopt (openRecvSocket)");
    }
  addr->sin_port = htons(FUMIED_CLIENT_PORT);
  if ( bind(*s, (struct sockaddr *)addr, sizeof(*addr))  < 0 )
    {
      error("bind (openRecvSocket)");
    }
}

int
rcvAndCheckFumiedMsg(long timeout)
{
  fd_set fdset;
  time_t prevTime;
  struct sockaddr_in addr;
  struct timeval tm;
  int len;
  struct ip *ip=(struct ip *)((struct udpiphdr *)UdpIpMsg.udpipmsg)->ip;

  bzero((char *)&UdpIpMsg, sizeof(udpipMessage));
  bzero((char *)&addr, sizeof(addr));
  tm.tv_sec = 0;
  time(&prevTime); 
  while ( timeout > 0 )
    {
      FD_ZERO(&fdset);
      FD_SET(fumiedSocket, &fdset);
      tm.tv_usec  = timeout;
      if ( select(fumiedSocket+1, &fdset, (fd_set *)NULL, (fd_set *)NULL, &tm) < 0 )
	{
	  debug("select (rcvAndCheckFumiedMsg)");
	  return 0;			/* receive unsuccessful */
	}
      len = sizeof(udpipMessage);
      if ( FD_ISSET(fumiedSocket, &fdset) )
	{
	  if ( recvfrom(fumiedSocket, (char *)&UdpIpMsg, len, 0,
			(struct sockaddr *)&addr, &len) < 0 )
	    {
	      debug("recvfrom (rcvAndCheckFumiedMsg)");
	      return 0;			/* receive unsuccessful */
	    }
	  FumiedMsgRecv = (struct fumiedMessage *)&UdpIpMsg.udpipmsg[(ip->ip_hl<<2)+sizeof(struct udphdr)];
	  if (*((unsigned long *)(FumiedMsgRecv->options)) == htonl(MAGIC_COOKIE))
	    return 1;
	}
      timeout -= (time(NULL) - prevTime)*1000000;
      time(&prevTime);
    }
  return 0;				/* receive timeout */
}

int
waitChkReXmitMsg(struct sockaddr *addr, int nretry)
{
  long tm;

  tm = getNextTimeout(INIT_TIMEOUT);
  while ( nretry-- )
    {
      if ( rcvAndCheckFumiedMsg(tm) )
	{
	  return 1;
	}
       if ( sendto(fumiedSocket,&UdpIpMsg,sizeof(struct ether_header)+
	       sizeof(udpiphdr)+sizeof(struct fumiedMessage),0,
	       addr,sizeof(struct sockaddr)) == -1 )
	{
	  error("sendto (waitChkReXmitMsg)");
	}
      tm = getNextTimeout(NEXT_TIMEOUT);
    }
  return 0;
}

void
setWaitMsgType(int type, u_int *ptype)
{
  *ptype |= (1 << (type-1));
}

int
if_register_socket(struct interface_info *info)
{
  struct sockaddr_in name;
  int sock;
  int flag;

  /* Set up the address we're going to bind to. */
  name.sin_family = AF_INET;
  name.sin_port = local_port;
  name.sin_addr.s_addr = INADDR_ANY;
  memset (name.sin_zero, 0, sizeof (name.sin_zero));

  /* Make a socket... */
  if ((sock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    error ("Can't create fumied socket: %m");

  /* Set the REUSEADDR option so that we don't fail to start if
     we're being restarted. */
  flag = 1;
  if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
		  (char *)&flag, sizeof flag) < 0)
    error ("Can't set SO_REUSEADDR option on fumied socket: %m");

  /* Set the BROADCAST option so that we can broadcast FUMIED responses. */
  if (setsockopt (sock, SOL_SOCKET, SO_BROADCAST,
		  (char *)&flag, sizeof flag) < 0)
    error ("Can't set SO_BROADCAST option on fumied socket: %m");

  /* Bind the socket to this interface's IP address. */
  if (bind (sock, (struct sockaddr *)&name, sizeof name) < 0)
    error ("Can't bind to fumied address: %m");

  /* Bind this socket to this interface. */
  if (setsockopt (sock, SOL_SOCKET, SO_BINDTODEVICE,
		  (char *)(info -> ifp), sizeof *(info -> ifp)) < 0)
    {
      error("setting SO_BINDTODEVICE");
    }

  return sock;
}

void
if_register_send(struct interface_info *info)
{
  info -> wfdesc = info -> rfdesc;
  note ("Sending on   Socket/%s", info -> name);

}

void
if_register_receive(struct interface_info *info)
{
  /* If we're using the socket API for sending and receiving,
     we don't need to register this interface twice. */
  info -> rfdesc = if_register_socket (info);
  note ("Listening on Socket/%s", info -> name);
}

ssize_t
send_packet(struct interface_info *interface, struct packet *packet,
	    struct fumiedMessage *raw, size_t len, struct in_addr from,
	    struct sockaddr_in *to, struct hardware *hto)
{
  int result;

  int retry = 0;
  do
    {
      result = sendto (interface -> wfdesc, (char *)raw, len, 0,
		       (struct sockaddr *)to, sizeof *to);
    }
  while ((to -> sin_addr.s_addr == htonl (INADDR_BROADCAST)) && result < 0
	 && (errno == EHOSTUNREACH || errno == ECONNREFUSED) && retry++ < 10);

  return result;
}

ssize_t
receive_packet(struct interface_info *interface, u_char *buf,
	       size_t len, struct sockaddr_in *from,
	       struct hardware *hfrom)
{
  int flen = sizeof *from;
  int result;

  int retry = 0;
  do
    {
      result = recvfrom (interface -> rfdesc, buf, len, 0,
			 (struct sockaddr *)from, &flen);
    }
  while (result < 0 && (errno == EHOSTUNREACH ||
			errno == ECONNREFUSED) && retry++ < 10);

  return result;
}
